/*******************************************************************************
 File:		lcd.h
 Compiler:	AVR-GCC 3.3
 Hardware:	AVR
 Date:		05.06.2009

 Basic routines for interfacing a HD44780U-based text LC-Display
*******************************************************************************/

#include <stdio.h>

// Clear enter display area, restore display from shift, and load address
// counter with DD RAM adress 0x00
#define LCD_CLR                  0x01

// Restore display from shift, and load address counter with DD RAM adress 0x00
#define LCD_HOME                 0x02

// Specify cursor advance direction and display shift mode
// This operation takes place after each data transfer
#define LCD_ENTRY_DEC            0x04	// cursor moves to the left
#define LCD_ENTRY_INC            0x06	// cursor moves to the right
#define LCD_ENTRY_DEC_SHIFT      0x05	// shift display to the left
#define LCD_ENTRY_INC_SHIFT      0x07	// shift display to the right

// Specify activation of display, cursor, and blinking of character
#define LCD_DISP_OFF             0x08	// display off
#define LCD_DISP_ON              0x0C	// display on
#define LCD_DISP_ON_CURSOR       0x0E	// display on, cursor on
#define LCD_DISP_ON_BLINK        0x0D	// display on, blink char
#define LCD_DISP_ON_CURSOR_BLINK 0x0F	// display on, cursor on, blink char

// Shift display or move cursor
#define LCD_MOVE_CURSOR_LEFT     0x10	// move cursor left  (decrement)
#define LCD_MOVE_CURSOR_RIGHT    0x14	// move cursor right (increment)
#define LCD_MOVE_DISP_LEFT       0x18	// shift display left
#define LCD_MOVE_DISP_RIGHT      0x1C	// shift display right


/*******************************************************************************
lcd_str: stream object to the LCD
Use with fprintf(&lcd_str, ...) or
redirect the standard output stream with stdout = &lcd_str; and use printf(...)
*******************************************************************************/
extern FILE lcd_str;

/*******************************************************************************
Initialize display, see HD44780 data sheet
After initialisation the display is off!
*******************************************************************************/
void LCD_init();


/*******************************************************************************
Load custom character. This function change the cursor position!
Input:	 pointer to array of char
		 character code (0x00 - 0x07)
*******************************************************************************/
void LCD_loadCustomChar(unsigned char* customCharArray, unsigned char charCode);


/*******************************************************************************
Send instruction to LCD unit
Input:	 Instruction code (use predefined codes from lcd.h)
*******************************************************************************/
void LCD_instruction(unsigned char);


/*******************************************************************************
Set cursor to specified position
Input:	 vertical position    (0: first line)
		 horizontal position  (0: left most position)
*******************************************************************************/
void LCD_setpos(unsigned char, unsigned char);


/*******************************************************************************
Display character at current cursor position
Input:	 character to be displayed
*******************************************************************************/
#define LCD_putchar(__c) LCD_putc(__c, NULL)


/*******************************************************************************
Display character at current cursor position
Function is used by stream object lcd_str
Input:	 character to be displayed
		 Pointer to FILE Object (shoud be NULL because Pointer is never used)
*******************************************************************************/
int LCD_putc(char, FILE *stream);


/*******************************************************************************
Display string without auto linefeed
Input:	 string to be displayed
*******************************************************************************/
void LCD_puts(const char *);
